@@ -3,7 +3,7 @@
 # est.map.R
 #
 # copyright (c) 2001-2011, Karl W Broman
-# last modified Jan, 2011
+# last modified Dec, 2011
 # first written Apr, 2001
 #
 #     This program is free software; you can redistribute it and/or
@@ -153,7 +153,7 @@ function(cross, chr, error.prob=0.0001, map.function=c("haldane","kosambi","c-f"
       one.map <- FALSE
       cfunc <- "est_map_4way"
     }
-    else if(type=="ri8sib" || type=="ri4sib" || type=="ri8self" || type=="ri4self") {
+    else if(type=="ri8sib" || type=="ri4sib" || type=="ri8self" || type=="ri4self" || type=="bgmagic16") {
       cfunc <- paste("est_map_", type, sep="")
       one.map <- TRUE
       if(chrtype[i] == "X")
